@@ -2,7 +2,10 @@ from ..utils import utils
 import numpy as np
 from pyswarms.single.global_best import GlobalBestPSO
 from .csip_access import ServiceClient
-import json
+from os import path
+import copy
+# import json
+import datetime
 
 
 def service(x, param, url, step_param_names, step_objfunc, calib_params, files=None):
@@ -45,7 +48,8 @@ def global_best(steps, rounds, args, n_particles, iters, options, rtol=1.0E-6, f
         options : dict
             PSO options (see pyswarms)
         rtol : float
-            tolerance to exit rounds (default: 1.0E-6)
+            relative error in objective_func(best_pos) acceptable for
+            convergence. Default is :code:`-np.inf`
         ftol : float
             PSO tolerance
 
@@ -73,9 +77,18 @@ def global_best(steps, rounds, args, n_particles, iters, options, rtol=1.0E-6, f
     best_cost = np.ones(len(steps))
     optimizer = np.empty(len(steps), dtype=object)
 
+    # trace of steps info
+    step_trace = {}
+    start_time =  datetime.datetime.now()
     for r in range(max_rounds):
         no_improvement = np.full(len(steps), True)
         for s, step in enumerate(steps):
+
+            # check if forced exit.
+            if path.exists("stop"):
+                print('\n>>>>> stop file found, exit now.')
+                break
+
             param_names, bounds, objfunc = utils.get_step_info(steps, s)
             # maybe clone args?
             args['step_param_names'] = param_names
@@ -83,13 +96,14 @@ def global_best(steps, rounds, args, n_particles, iters, options, rtol=1.0E-6, f
             # get calibrated parameter from all other steps
             args['calib_params'] = utils.get_calibrated_params(steps, s)
 
+            # create optimizer in the first round.
             if optimizer[s] is None:
                 optimizer[s] = GlobalBestPSO(n_particles, len(param_names),
                                              options=options, bounds=bounds, ftol=ftol)
-
             print('\n>>>>> R{}/S{}  particle params: {}  calibrated params: {}\n'.format(r + 1, s + 1, param_names,
                                                                              args['calib_params']))
 
+            # perform optimization
             cost, pos = optimizer[s].optimize(service, iters=iters, **args)
 
             # capture the best cost
@@ -100,6 +114,9 @@ def global_best(steps, rounds, args, n_particles, iters, options, rtol=1.0E-6, f
 
             print('\n     Step summary, best particle values: {} '.format(pos))
 
+            key = "r{}s{}".format(r+1, s+1)
+            step_trace[key]= copy.deepcopy(steps)
+
             # print(json.dumps(steps, sort_keys=False, indent=2))
 
         round_cost = np.sum(best_cost)
@@ -109,5 +126,13 @@ def global_best(steps, rounds, args, n_particles, iters, options, rtol=1.0E-6, f
         if (r + 1 >= min_rounds) and all(no_improvement):
             break
 
-    print('Done after {} out of {} rounds'.format(r + 1, max_rounds))
-    return optimizer, cost  # return the last output
+    end_time = datetime.datetime.now()
+    print('Done in {} after {} out of {} rounds'.format(str(end_time - start_time), r + 1, max_rounds))
+
+    step_trace['rounds'] = r+1
+    step_trace['steps'] = len(steps)
+    step_trace['iters'] = iters
+    step_trace['particles'] = n_particles
+    step_trace['time'] = str(end_time - start_time)
+
+    return optimizer, cost, step_trace  # return the last output
